package com.kakao.sdk.partner.auth.model

import android.os.Parcelable
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.KakaoSdkError
import kotlinx.android.parcel.Parcelize
import java.lang.RuntimeException

/**
 * 카카오 AgeAuth API 호출 시 에러 응답
 *
 * @property status //todo
 * @property msg //todo
 */
/**
 * SDK 내에서 발생하는 클라이언트 에러
 */
@Parcelize
data class AgeAuthError(
    val status: Int,
    val msg: String
) : RuntimeException(msg), Parcelable



@Target(AnnotationTarget.PROPERTY)
annotation class Description (
    val value : String
)


// 연령인증(실명/본인/성인 인증) 에러 종류 입니다.
enum class AgeAuthErrorCause(val status: Int) {

    /// 인증되지 않은 사용자일 경우
    @Description("user is unauthenticated.")
    UNAUTHORIZED(-401),

    /// 파라미터 형식이 잘못됨
    @Description("client information is incompatible.")
    BAD_PARAMETERS(-440) ,

    /// 연령인증이 되지 않아서 연령인증이 필요한 상황
    /// (보통 사용자가 외부 연령인증 페이지까지 갔다가 잘못된 정보를 입력하거나 취소하는 경우 발생할 수 있음)
    @Description("age verification is required but not completed. (Normal situation to proceed age verification")
    NOT_AUTHORIZED_AGE(-450),

    /// 현재 앱의 연령제한보다 사용자의 연령이 낮은 경우
    /// 대부분 만 14세 이상이지만 앱의 제한연령보다는 낮은 나이로 인증받은 경우 계정 DB에 인증 정보는 저장되지만, 서비스 이용은 불가하다는 페이지가 노출됩니다.
    /// *만 14세 미만은 인증 자체가 불가하도록 막았으므로, 이 오류코드가 리턴되지 않음.
    @Description("user is younger than age limit specified in current app.")
    LOWER_AGE_LIMIT(-451),

    /// 이미 연령인증이 되었습니다.
    /// (2차인증이 완료되어 있는 상황에서, 1차인증을 시도했을 경우에도 발생)
    @Description("age verification has already been completed. (Even after completing 2-step verification,  user attempts 1-step verification.")
    ALREADY_AGE_AUTHORIZED(-452),

    /// 연령인증 횟수 초과
    @Description("user has exceeded the maximum number of failed attempts at age verification.")
    EXCEED_AGE_CHECK_LIMIT(-453),

    /// 이전에 인증했던 정보와 불일치 (생일)
    /// (기존 BIRTHDAY_MISMATCH 에서 수정됨)
    @Description("does not match with the previously authenticated information(birthday). (replaced from BIRTHDAY_MISMATCH)")
    AGE_AUTH_RESULT_MISMATCH(-480),

    /// CI 정보가 불일치할 경우
    @Description("CI information does not match.")
    CI_RESULT_MISMATCH(-481),


    /// 기타 서버 에러
    @Description("failed to find user, birthday does not match with the birthday received from CA, or unexpected error occurs.")
    ERROR(-500),

    /// 알수 없는 에러
    @Description("unknown error.")
    UNKNOWN(-9999);
}