package com.kakao.sdk.partner.user.network

import com.kakao.sdk.auth.AuthApiClient
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.model.ApiErrorCause
import com.kakao.sdk.common.model.ApiErrorResponse
import com.kakao.sdk.common.model.ApplicationContextInfo
import com.kakao.sdk.common.util.KakaoJson
import com.kakao.sdk.partner.auth.verifyAge
import okhttp3.Interceptor
import okhttp3.Response
import okhttp3.ResponseBody
import java.util.concurrent.CountDownLatch

class RequiredAgeVerificationInterceptor(
    private val contextInfo: ApplicationContextInfo = KakaoSdk.applicationContextInfo)
    : Interceptor {
    override fun intercept(chain: Interceptor.Chain): Response {
        val originalResponse = chain.proceed(chain.request())

        if (originalResponse.isSuccessful) {
            return originalResponse
        }

        val body = originalResponse.body()
        val bodyString = body?.string()

        val response = originalResponse.newBuilder()
            .body(ResponseBody.create(body?.contentType(), bodyString))
            .build()

        val apiErrorResponse = bodyString?.let {
            KakaoJson.fromJson<ApiErrorResponse>(it, ApiErrorResponse::class.java)
        }
        val apiErrorCause = apiErrorResponse?.let {
            KakaoJson.fromJson<ApiErrorCause>(it.code.toString(), ApiErrorCause::class.java)
        }

        // 405 연령인증 필요
        if (apiErrorCause == ApiErrorCause.RequiredAgeVerification) {
            var error: Throwable? = null
            val latch = CountDownLatch(1)

            AuthApiClient.instance.verifyAge(context = contextInfo.applicationContext) { e ->
                error = e; latch.countDown()
            }

            latch.await()
            error?.let {
                throw java.io.IOException(error)
            } ?:  return chain.proceed(response.request())
        }
        return response
    }
}
